'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _fluxStandardAction = require('flux-standard-action');

var _errors = require('./errors');

var _errors2 = _interopRequireDefault(_errors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function validateInput(_ref, options) {
  var typePrefix = _ref.typePrefix,
      types = _ref.types,
      callAPI = _ref.callAPI,
      _ref$formatData = _ref.formatData,
      formatData = _ref$formatData === undefined ? function (res) {
    return res;
  } : _ref$formatData,
      _ref$shouldCallAPI = _ref.shouldCallAPI,
      shouldCallAPI = _ref$shouldCallAPI === undefined ? function () {
    return true;
  } : _ref$shouldCallAPI,
      _ref$payload = _ref.payload,
      payload = _ref$payload === undefined ? {} : _ref$payload,
      _ref$meta = _ref.meta,
      meta = _ref$meta === undefined ? {} : _ref$meta;

  if (options === undefined && (!Array.isArray(types) || types.length !== 3 || !types.every(function (t) {
    return typeof t === 'string';
  }))) {
    throw new Error(_errors2['default'].types);
  }
  if (options !== undefined && typeof typePrefix !== 'string') {
    throw new Error(_errors2['default'].type);
  }
  if (typeof callAPI !== 'function') {
    throw new Error(_errors2['default'].callAPI);
  }
  if (typeof formatData !== 'function') {
    throw new Error(_errors2['default'].formatData);
  }
  if (typeof shouldCallAPI !== 'function') {
    throw new Error(_errors2['default'].shouldCallAPI);
  }
  if (typeof payload !== 'object') {
    throw new Error(_errors2['default'].payload);
  }
  if (typeof meta !== 'object') {
    throw new Error(_errors2['default'].meta);
  }
}

function validateOptions(_ref2) {
  var pendingSuffix = _ref2.pendingSuffix,
      successSuffix = _ref2.successSuffix,
      errorSuffix = _ref2.errorSuffix;

  if (typeof pendingSuffix !== 'string' || typeof successSuffix !== 'string' || typeof errorSuffix !== 'string') throw new Error(_errors2['default'].options);
}

function optionsAreValid(typePrefix, types, options) {
  if (options === undefined && !types) return false;
  if (options === undefined && types) return true;
  if (options !== undefined && !typePrefix) return false;
  if (options !== undefined && typePrefix) {
    validateOptions(options);
    return true;
  }
  return false;
}

function getActionTypes(typePrefix, types, options) {
  if (options !== undefined) {
    return ['' + typePrefix + options.pendingSuffix, '' + typePrefix + options.successSuffix, '' + typePrefix + options.errorSuffix];
  }
  return types;
}

function createSlimAsyncMiddleware(options) {
  return function (_ref3) {
    var dispatch = _ref3.dispatch,
        getState = _ref3.getState;
    return function (next) {
      return function (action) {
        var typePrefix = action.typePrefix,
            types = action.types,
            callAPI = action.callAPI,
            _action$formatData = action.formatData,
            formatData = _action$formatData === undefined ? function (res) {
          return res;
        } : _action$formatData,
            _action$shouldCallAPI = action.shouldCallAPI,
            shouldCallAPI = _action$shouldCallAPI === undefined ? function () {
          return true;
        } : _action$shouldCallAPI,
            _action$payload = action.payload,
            payload = _action$payload === undefined ? {} : _action$payload,
            _action$meta = action.meta,
            meta = _action$meta === undefined ? {} : _action$meta;

        var isFSACompliant = options && options.isFSACompliant === false ? false : true;
        if (typePrefix && !callAPI) return next(action);
        if (!optionsAreValid(typePrefix, types, options)) return next(action);

        validateInput(action, options);

        if (!shouldCallAPI(getState())) return Promise.resolve(getState());

        var _getActionTypes = getActionTypes(typePrefix, types, options),
            pendingType = _getActionTypes[0],
            successType = _getActionTypes[1],
            errorType = _getActionTypes[2];

        var pendingAction = isFSACompliant ? { payload: payload, type: pendingType } : _extends({}, payload, { type: pendingType });

        if (isFSACompliant && !(0, _fluxStandardAction.isFSA)(pendingAction)) next(action);else dispatch(pendingAction);

        return callAPI().then(function (response) {
          var formattedData = formatData(response);
          if (typeof formattedData !== 'object') {
            throw new Error(_errors2['default'].formatDataReturn);
          }

          var successAction = isFSACompliant ? {
            type: successType,
            payload: _extends({}, payload, formattedData),
            meta: meta
          } : _extends({
            type: successType
          }, formattedData, {
            meta: meta
          });

          if (isFSACompliant && !(0, _fluxStandardAction.isFSA)(successAction)) next(action);else dispatch(successAction);

          return Promise.resolve(getState());
        })['catch'](function (error) {
          var errorAction = isFSACompliant ? {
            payload: error,
            error: true,
            type: errorType,
            meta: meta
          } : {
            message: error.message,
            error: true,
            type: errorType,
            meta: meta
          };

          if (isFSACompliant && !(0, _fluxStandardAction.isFSA)(errorAction)) next(action);else dispatch(errorAction);

          return Promise.reject(error);
        });
      };
    };
  };
}

var slimAsync = createSlimAsyncMiddleware();
slimAsync.withOptions = createSlimAsyncMiddleware;

exports['default'] = slimAsync;